#!/usr/bin/python
import sys,os,getopt
import timeit
command = '-d ./test_data/tmp -r ./test_data/GCF_000016525.1_ASM1652v1_genomic.fna -t fastq -1 ./test_data/NC_009515.1_0.01_100_400_30.0a70.0_0.001.read1.fastq -2 ./test_data/NC_009515.1_0.01_100_400_30.0a70.0_0.001.read2.fastq -l 100 -c 100 -g 1853160'



##################################################################################################################

def calculate_best_kmer(coverage,readLen,genomeLen):
    #x0 = coverage
    #x1 = readLen
    #x2 = genomeLen

    ployFeatures = ['x0', 'x1', 'x2', 'x0^2', 'x0 x1', 'x0 x2', 'x1^2', 'x1 x2', 'x2^2']

    coef = [-2.89973043e-01,  2.90012710e-01,  3.12264949e-05, -1.18792202e-04,
        3.98041097e-03, -4.82394282e-09, -6.19137730e-04, -5.62306677e-09,
       -5.47151213e-12]


    intercept = -3.543848637743409


    y = coverage*coef[0] + readLen*coef[1] + genomeLen*coef[2] + coef[3]*(coverage**2) + coef[4]*coverage*readLen+\
        coef[5]*coverage*genomeLen + coef[6]*(readLen**2)+coef[7]*readLen*genomeLen+ coef[8]*(genomeLen**2)+intercept

    return int(round(y,0))


##################################################################################################################

t = timeit.Timer()
times = []
STARTTIME = t.timer()
times.append(('start',t.timer()))



#load input parameter
try:
    opts,args=getopt.getopt(sys.argv[1:],'d:r:t:1:2:l:c:g:')
    for i in opts:
        if i[0]=='-d':
            resName = i[1]
        if i[0]=='-r':
            refLoc = i[1]
        if i[0]=='-t':
            refType = i[1]
        if i[0]=='-1':
            forwardSeq = i[1]
        if i[0]=='-2':
            reverseSeq = i[1]
        # if i[0]=='-x':
        #     kmer = int(i[1])
        if i[0]=='-l':
            readLen = int(i[1])
        if i[0]=='-c':
            coverage = int(i[1])
        if i[0]=='-g':
            genomeLen = int(i[1])


except:
    print ('Please check your input parameters')
    sys.exit(0)



currentPath = os.path.dirname(os.path.abspath(__file__))

resLoc = os.path.abspath(resName)
command = 'mkdir '+resLoc
os.system(command)

codeLoc = currentPath + '/src'
genomeLoc = os.path.abspath(refLoc)
forwardSeq = os.path.abspath(forwardSeq)
reverseSeq = os.path.abspath(reverseSeq)



#################################
'''
Velvet kmer length limit
it must be an odd number, to avoid palindromes. If you put in an even number, Velvet will just decrement it and proceed
it must be below or equal to the MAXKMERLENGTH (by default = 31bp) variable in the global.h file, because of the use of 
fixed-length arrays.it must be strictly inferior to read length, otherwise you simply will not observe any overlaps 
between reads, for obvious reasons.
'''
##################################
#calcualte the best k-mer
bestKmer = calculate_best_kmer(coverage, readLen, genomeLen)

bestKmerRange = [-1, -1]
if bestKmer % 2 == 0:
    bestKmer = bestKmer - 1

if bestKmer > 85:
    bestKmer = 85

bestKmerRange[0] = bestKmer - 2
bestKmerRange[1] = bestKmer + 2

#start to code, Velvet only accept odd numbe
# for k in np.arange(minKmer,maxKmer+1,2):
for k in range(bestKmerRange[0], bestKmerRange[1] + 2, 2):
    kLoc = resLoc+'/'+str(k)
    command = 'mkdir ' + kLoc
    os.system(command)

    #Create the graph data structure with velveth
    command1 = ' '.join([codeLoc+'/velveth_ucrt_90_openmp',kLoc,str(k),'-'+refType,'-shortPaired','-separate',forwardSeq,reverseSeq])
    print command1
    os.system(command1)
    times.append(('kmer '+str(k)+':create graph',t.timer()))

    #To get the actual contigs you will have to run velvetg on the created graph
    command2 = ' '.join([codeLoc+'/velvetg_ucrt_90_openmp',kLoc])
    print command2
    os.system(command2)
    times.append(('kmer '+str(k)+':get contigs',t.timer()))

    #blast aligner
    align_out =resLoc + '/align_ref-contigs_'+str(k)+'.pslx'
    command3 = ' '.join([codeLoc+'/aligner',genomeLoc,kLoc+'/uncorrectedContigs.fa',align_out,'-out=pslx'])
    os.system(command3)
    times.append(('kmer '+str(k)+':aligner',t.timer()))

    # #run
    command4 = ' '.join([codeLoc+'/main.py',resLoc,str(k),'>',resLoc+'/mainLog_'+str(k)])
    os.system(command4)
    times.append(('kmer '+str(k)+':main procedure',t.timer()))


    # #remove the big files which only used in debug
    removeLoc = resLoc+'/'+str(k)
    command5 = 'rm -r '+removeLoc
    os.system(command5)


times.append(('finished k-mer', t.timer()))


#run combining result
tmpKmer = ' '.join([str(kmerLength) for kmerLength in range(bestKmerRange[0],bestKmerRange[1]+2,2)])
command = ' '.join(['python2.7',codeLoc + '/CombinePredictions_selfKmer.py',resLoc,resLoc,tmpKmer])
os.system(command)
times.append(('finished', t.timer()))

#write total running time into file
ENDTIME = t.timer()
times.append(('total',str(ENDTIME-STARTTIME)))
resName = resLoc + '/total_time'
with open('%s' %resName,'w') as f:
    for item in times:
        tmp = '\t'.join([str(one) for one in item])
        f.write('%s\n' %tmp)


#delete all the intermediate files
from os import listdir
from os.path import isfile,join
#delete velvet result
tmpNames = [item for item in listdir('%s' %resLoc) if isfile(join('%s' %resLoc,item)) and (item.startswith('EM') or
            item.startswith('align') or item.startswith('main') or item.startswith('predictions'))]
map(lambda name: os.remove(join('%s' %resLoc,name)), tmpNames)


print 'done'
