#!/usr/bin/python
import networkx as nx
import Statistics
import sys
import Constants
import operator


def FindMismatches(startPos, seq, genomeFragment):
    mismatchList = {}
    for i in range(0, len(seq)):
        if (seq[i] != genomeFragment[i]):
            mismatchList[startPos + i] = seq[i]
    
    return mismatchList
  

def ConvertBlatAlignedNodesToNX(blatFile, kmer):
    nxGraph = nx.DiGraph()
    with open (blatFile) as f:
        for _ in xrange(5):
            next(f)
        for line in f:
            splitted = line.split("\t")
            if (len(splitted) < 10):
                continue
            
            mismatchCnt = int(splitted[1])
            qryGapsCnt = int(splitted[5])
            gnmGapsCnt = int(splitted[7])
            strand = splitted[8]
            matchPercentage = float(splitted[0]) / float(splitted[10])        
            startPos = int(splitted[15])     #start pos in ref
            endPos = int(splitted[16])  #end pos in ref
            seq = splitted[21]
            genomeFragment = splitted[22]
            nodeName = splitted[9]
            # high match percentage, and don't allow insert
            if (matchPercentage < 0.95 or qryGapsCnt > 0 or gnmGapsCnt > 0):
                continue
            # remove the \n in the genome
            genomeFragment = genomeFragment[0:-1]
                    
            
            mismatches = FindMismatches(startPos, seq, genomeFragment)

            splt = nodeName.split("_")
            attr_dict = {}
            nodeId = int(splt[1])
            # if nodeId==4055:
            #     print 'here'
            ln = int(splt[3])
            cov = float(splt[5])
            attr_dict['len'] = ln
            attr_dict['cov'] = cov
            attr_dict['start'] = startPos
            #bug fixed
            #attr_dict['end'] = startPos + ln - 1
            attr_dict['end'] = endPos
            attr_dict['seq'] = seq
            attr_dict['mismatches'] = mismatches

            # # if attr_dict['start']>=224273 or attr_dict['end']<=224177:
            # if attr_dict['start'] >= 20167 or attr_dict['end'] <= 20109:
            #     pass
            # else:
            #     print 'here'

            if (ln >= kmer and cov >= Constants.COVERAGE_THRESHOLD):
                #nxGraph.add_node(nodeId, attr_dict)
                nxGraph.add_nodes_from([(nodeId, attr_dict)])

    #sort the node by end position
    endPositions = nx.get_node_attributes(nxGraph, 'end')
    sortedEnds = sorted(endPositions.items(), key = operator.itemgetter(1)) #sortedEnds = [(nodeID,endPositions),()...]
    nodesOrder = [item[0] for item in sortedEnds]   
    nodesCount = len(nodesOrder)

    #group the nodes by two nodes overlaped and has distinct mismatches.
    i = 0
    groupedNodes = []
    pNodeIndices = []
    while(i < nodesCount):
        n = nodesOrder[i]
#         s1, e1 = nxGraph.node[n]['start'], nxGraph.node[n]['end']
        parallelNodes = [n]
        while (True):
            i += 1
            if (i >= nodesCount):               
                groupedNodes.append(parallelNodes)
                if (len(parallelNodes) > 1):
                    pNodeIndices += parallelNodes
                break
            n = nodesOrder[i]
            s2 = nxGraph.node[n]['start']
            inserted = False
            for pn in parallelNodes:
#                 print n, pn
                #bug fixed
                #e1 = nxGraph.node[n]['end']
                e1 = nxGraph.node[pn]['end']
                if (s2 <= e1):
                    intersectionStart = s2
                    intersectionEnd = e1               
                    m1 = nxGraph.node[pn]['mismatches']
                    m2 = nxGraph.node[n]['mismatches']
#                     print m1, m2
                    #Mismatches in intersection
                    m1 = {pos:nucl for pos, nucl in m1.iteritems() if pos >= intersectionStart and pos <= intersectionEnd} 
                    m2 = {pos:nucl for pos, nucl in m2.iteritems() if pos >= intersectionStart and pos <= intersectionEnd}
#                     print intersectionStart, intersectionEnd
#                     print m1, m2
                    m1 = set([str(pos) + ":" + nucl for pos, nucl in m1.iteritems()])
                    m2 = set([str(pos) + ":" + nucl for pos, nucl in m2.iteritems()])
                    #exclude the mismatch in both, ?may be problem
                    sd1 = m1.symmetric_difference(m2)
                    sd2 = m2.symmetric_difference(m1)


                    # if len(m1)!=0 and len(m2)!=0:
                    #     print 'here'
                    #two node are overlapped and with disinct mismatch
                    if (len(sd1) > 0 or len(sd2) > 0):
                        inserted = True
                        parallelNodes.append(n)
                        break
            if (inserted == False):
                groupedNodes.append(parallelNodes)
                if (len(parallelNodes) > 1):
                    pNodeIndices += parallelNodes
                break 
    #add edges between group from
    for crt in range(0, len(groupedNodes) - 1):
        nxt = crt + 1
        for u in groupedNodes[crt]:
            for v in groupedNodes[nxt]:
                nxGraph.add_edge(u, v)




    # #write total mismatch into file
    # resLoc = '/media/student/study_working/project/project10/BHap/res/SRR1585343/'+str(kmer)
    #
    # resName = resLoc+'_totalMismatch'
    # mismatch = {}
    # for i in nxGraph.nodes:
    #     node = nxGraph.node[i]
    #     if len(node['mismatches'])!=0:
    #         for loc,nucl in node['mismatches'].iteritems():
    #             if loc in mismatch:
    #                 mismatch[loc].add(nucl)
    #             else:
    #                 mismatch.update({loc:set([nucl])})
    # with open('%s' %resName,'w') as f:
    #     for key,value in mismatch.iteritems():
    #         item = ','.join(list(value))
    #         tmp = '\t'.join([str(key),item])
    #         f.write('%s\n' %tmp)
    # print 'here'
    #
    #
    # #write segrageting sites into file
    # resName = resLoc+'_rawSites'
    # start = []
    # end = []
    # mismatch = []
    # nodeIDList = []
    # for group in groupedNodes:
    #     if len(group)>1:
    #         startList = []
    #         endList = []
    #         mismatcheList = []
    #         IDList = []
    #         for ID in group:
    #             s = nxGraph.node[ID]['start']
    #             e = nxGraph.node[ID]['end']
    #             m1 = nxGraph.node[ID]['mismatches']
    #             m1 = set([str(pos) + ":" + nucl for pos, nucl in m1.iteritems()])
    #             if len(m1)==0:
    #                 tmp = '.'
    #             else:
    #                 tmp = ','.join(m1)
    #             startList.append(s)
    #             endList.append(e)
    #             mismatcheList.append(tmp)
    #             IDList.append(ID)
    #
    #         start.append(startList)
    #         end.append(endList)
    #         mismatch.append(mismatcheList)
    #         nodeIDList.append(IDList)
    #
    # with open('%s' %resName,'w') as f:
    #     for i in range(len(start)):
    #         group_start = start[i]
    #         group_end = end[i]
    #         group_mismatch = mismatch[i]
    #         group_ID = nodeIDList[i]
    #         tmp1 = '|'.join([str(one) for one in group_start])
    #         tmp2 = '|'.join([str(one) for one in group_end])
    #         tmp3 = '|'.join([str(one) for one in group_mismatch])
    #         tmp4 = '|'.join([str(one) for one in group_ID])
    #         tmp = '\t'.join([tmp1,tmp2,tmp3,tmp4])
    #         f.write('%s\n' %tmp)

    pCovs = []
    for n in pNodeIndices:
        pCovs.append(nxGraph.node[n]['cov'])
#     print pCovs           
    return nxGraph, pCovs, pNodeIndices

def ConvertAlignedNodesToNX(samFile, kmer):
    nxGraph = nx.DiGraph()

    with open (samFile) as f:
        for _ in xrange(3):
            next(f)
        for line in f:
            splitted = line.split("\t")
            if (len(splitted) < 10):
                continue
                   
            startPos = int(splitted[3])
            nodeName = splitted[0]
            seq = splitted[9]
            mapq = int(splitted[4])
            mapped = True
            if splitted[2] == "*" or mapq < Constants.MAPQ_THRESHOLD:
                mapped = False
    
            if mapped == True: 
                splt = nodeName.split("_")
                attr_dict = {}
                nodeId = int(splt[1])
                ln = int(splt[3])
                cov = float(splt[5])
                attr_dict['len'] = ln
                attr_dict['cov'] = cov
                attr_dict['start'] = startPos
                attr_dict['end'] = startPos + ln - 1
                attr_dict['seq'] = seq
     
                if (ln >= kmer):
                    nxGraph.add_node(nodeId, attr_dict)

    
    endPositions = nx.get_node_attributes(nxGraph, 'end')
    sortedEnds = sorted(endPositions.items(), key = operator.itemgetter(1))
    nodesOrder = [item[0] for item in sortedEnds]   
    nodesCount = len(nodesOrder)
    i = 0
    groupedNodes = []
    while(i < nodesCount):
        n = nodesOrder[i]
        s1, e1 = nxGraph.node[n]['start'], nxGraph.node[n]['end']
        parallelNodes = [n]     
        while (True):
            i += 1
            if (i >= nodesCount):              
                groupedNodes.append(parallelNodes)
                break
            n = nodesOrder[i]
            s2, e2 = nxGraph.node[n]['start'], nxGraph.node[n]['end']
            if (s2 <= e1):
                parallelNodes.append(n)
            else:
                groupedNodes.append(parallelNodes)
                break 
#     print groupedNodes
    
    for crt in range(0, len(groupedNodes) - 1):
        nxt = crt + 1
        for u in groupedNodes[crt]:
            for v in groupedNodes[nxt]:
                nxGraph.add_edge(u, v)
        

    return nxGraph
    
            

def ConvertVelvetGraphToNX(graphFile):
    f = open(graphFile)
    firstLine = f.readline()
    firstLine = firstLine.strip()
    splt = firstLine.split("\t")
    nodesCount = int(splt[0])
    seqsCount = int(splt[1])
    kmerLen = int(splt[2])
    
    nxGraph = nx.DiGraph()
    
    removedNodes = []
    
    line = f.readline().strip()
    while (line.startswith("NODE")):
        splt = line.split("\t")
        attr_dict = {}
        nodeId = int(splt[1])
        ln = int(splt[2])
        kmercov = int(splt[3])
        attr_dict['len'] = ln
        attr_dict['cov'] = kmercov / ln
        attr_dict['contig'] = f.readline().strip()
        attr_dict['contigR'] = f.readline().strip()
        
#         print ln       
        if (ln >= kmerLen):
            nxGraph.add_node(nodeId, attr_dict)
        else:
            removedNodes.append(nodeId)
        
        line = f.readline().strip()    
        
    while (line.startswith("ARC")):
        splt = line.split("\t")
        src = int(splt[1])
        dest = int(splt[2])
        mult = int(splt[3])

        if (src < 0 and dest < 0):
            temp = src
            src = abs(dest)
            dest = abs(temp)
        else:
            src = abs(src)
            dest = abs(dest)
        
        if (src not in removedNodes and dest not in removedNodes):
            nxGraph.add_edge(src, dest, multiplicity=mult)
        
        line = f.readline().strip()
        
    pNodes = []
    for n in nxGraph.nodes_iter(data=False):        
        if nxGraph.out_degree(n) > 1:
            p = nxGraph.neighbors(n)
            pNodes += p
    pNodes = list(set(pNodes))
    pCovs = []
    for n in pNodes:
        pCovs.append(nxGraph.node[n]['cov'])
    
    return nxGraph, pCovs



def GetGraphNodeWeights(graph):
    weightList = []
    for nid, attr in graph.nodes_iter(data=True):
        weightList.append(attr['cov'])
    return weightList

#add start and end nodes
def AddSTtoGraph(graph):
    srcId = Constants.SRC_ID
    trgtId = Constants.TRGT_ID
    # graph.add_node(srcId, {'len' : 60, 'cov' : 1000, 'seq' : "", 'start' : 0, 'end' : 59})
    # graph.add_node(trgtId, {'len' : 60, 'cov' : 1000, 'seq' : "", 'start' : 100000000, 'end' : 100000000 + 59})
    start_attr = {'len' : 60, 'cov' : 1000, 'seq' : "", 'start' : 0, 'end' : 59}
    end_attr = {'len' : 60, 'cov' : 1000, 'seq' : "", 'start' : 100000000, 'end' : 100000000 + 59}
    graph.add_nodes_from([(srcId, start_attr)])
    graph.add_nodes_from([(trgtId, end_attr)])
    #bug fixed
    #for nid in graph.nodes_iter(data=False):
    for nid in graph.nodes(data=False):
        if (graph.in_degree(nid) == 0 and nid != srcId and nid != trgtId): #this node does not have incoming edges
            graph.add_edge(srcId, nid)
            #print "node without incoming edge:"+str(nid)
#         print "%%%%", graph.out_degree(srcId)
        if (graph.out_degree(nid) == 0 and nid != srcId and nid != trgtId): #this node does not have outgoing edges
            graph.add_edge(nid, trgtId)    
            #print "node without outgoing edge:"+str(nid)
    #print 'here'


def TrueCovProb(M, remainingCapacity):
    maxProb = 0
    for mi in M:
        prob = Statistics.Poisson(mi, remainingCapacity)
#         prob = Statistics.Normal(mi, 10, remainingCapacity)
        if prob > maxProb:
            maxProb = prob

    zeroProb = Statistics.Normal(0, 3, remainingCapacity)
    if zeroProb > maxProb:
        maxProb = zeroProb
               
    return maxProb

def CalculateNodeCost(graph, M, flow):
    #M is the remain flow, flow is current flow, graph is all nodes.
    costDict = {}
    #bug fixed here
    #for nid, attr in graph.nodes_iter(data=True):
    for nid, attr in graph.nodes(data=True):
        capacity = attr['cov']
        remainingCapacity = capacity - flow
        cost = 1 - TrueCovProb(M, remainingCapacity)
        costDict[nid] = cost
    #fix bugs
    #nx.set_node_attributes(graph, 'cost', costDict)
    nx.set_node_attributes(graph, costDict, 'cost')
    #print 'here'


def FindLowCostPath(graph, flow):
    costs = nx.get_node_attributes(graph, 'cost')
    covs = nx.get_node_attributes(graph, 'cov')
    curNode = Constants.SRC_ID

    pathTotalCost = 0
    nodesInPath = []
    while (curNode != Constants.TRGT_ID):
        #bug fix
        #nghbr = graph.neighbors(curNode)
        nghbr = [n for n in graph.neighbors(curNode)]

        #fix bug the end node in the list! may need to be fixed
        if len(nghbr)==1 and nghbr[0]==Constants.TRGT_ID:
            break

        if (len(nghbr) == 1):
            curNode = nghbr[0]
        #if current node has more than one next nodes, then pick the node with lowest cost as next node.
        elif (len(nghbr) > 1):
            minCost = sys.maxint
            minNode = -1
            for n in nghbr:
                if (costs[n] < minCost):
                    minCost = costs[n]
                    minNode = n
            curNode = minNode

        if curNode==Constants.TRGT_ID:
            print 'here'


        nodesInPath.append(curNode)
        curCost = costs[curNode]
        pathTotalCost +=  curCost
    
    return nodesInPath, pathTotalCost

#check whether it is almost zero or not
def IsAlmostZero(x):
    zeroProb = Statistics.Normal(0, 1, x)
    if zeroProb > Constants.ZERO_PROB_THRESHOLD:
        return True
    return False      

def ReduceFlowFromPath(graph, path, flow):#, parallelNodes):
    cnt = 0
    for n in path:
#         if (n in parallelNodes):
#             continue
        curCapacity = graph.node[n]['cov']
        curCapacity -= flow

        if (IsAlmostZero(curCapacity)): # The node capacity decreased down to zero so remove it
            graph.remove_node(n)
        else:
            graph.node[n]['cov'] = curCapacity
            
    
def FindMismatchingNodes(graph):
    mismatchingNodes = []
    for nid, attr in graph.nodes_iter(data=True):
        if (nid != Constants.SRC_ID and nid != Constants.TRGT_ID and len(attr['mismatches']) > 0):
            mismatchingNodes.append(nid)
    return mismatchingNodes
 
