#!/usr/bin/python
import Network
import networkx as nx
from sys import argv
import EM
import copy
import numpy as np


if (len(argv) != 3):
    print "usage:"
    print "main.py ExperimentDirectory kmer"
    exit()

expDir = argv[1]
kmer = int(argv[2])


#uncrrGraphFile = expDir + "/UncorrectedGraph"
blatFile = expDir + "/align_ref-contigs_" + str(kmer) + ".pslx"
emOut = expDir + "/EM_" + str(kmer)
predictionsFile = expDir + "/predictions_" + str(kmer)


def main():
    #add edges, grouped nodes and corresponding coverage.
    graph, pCovs, pNodeIndices = Network.ConvertBlatAlignedNodesToNX(blatFile, kmer)
    print nx.number_of_nodes(graph), nx.number_of_edges(graph)


    # #save segregating sites into files
    # resName = expDir+'segregating_sites_'+str(kmer)
    # with open('%s' %resName,'w') as f:
    #     for node in pNodeIndices:
    #         item = graph.node[node]
    #         mismatches = ','.join([str(key)+':'+value.upper() for key,value in item['mismatches'].iteritems()])
    #         tmp = '\t'.join([str(item['start']),str(item['end']),str(item['cov']),str(item['len']),mismatches])
    #         f.write('%s\n' %tmp)


    #XIN fixed bug
    from scipy.stats import norm
    quantile95 = norm.ppf([0.025,0.975],np.mean(pCovs),np.std(pCovs))
    tempCovs = sorted([item for item in pCovs if item >= quantile95[0]
                       and item <= quantile95[1]])

    # #option2
    # tempCovs = sorted(pCovs)

    # original
    # covMedian = numpy.median(sorted(pCovs))
    # print "median", covMedian
    # covThreshold = covMedian * 2
    # tempCovs = [item for item in pCovs if item <= covThreshold]


    # #save EM input into file
    # import pickle
    # resName = expDir+'EM_input_'+str(kmer)+'.pkl'
    # fPKL = open('%s' %resName,'wb')
    # pickle.dump(tempCovs,fPKL)
    # fPKL.close()
    #
    # resName = expDir+'EM_input_'+str(kmer)+'.txt'
    # with open('%s' %resName,'w') as f:
    #     for item in tempCovs:
    #         f.write('%s\n' %str(item))
    #
    # import pickle
    # inName = expDir+'EM_input_63.pkl'
    # fPKL = open('%s' %inName,'rb')
    # tempCovs1 = pickle.load(fPKL)
    # fPKL.close()

    #calculate EM
    M,alpha = EM.EstimateParameters(tempCovs)
    fm = open(emOut, "a")
    #fm.write("\n".join([str(m) for m in M]))
    fm.write("\n".join([str(M[i])+'\t'+str(alpha[i]) for i in range(len(M))]))
    fm.write("\n****************************************\n\n\n\n")
    fm.close()


    # #restore EM from previous calculated
    # inName = expDir + '/EM_'+str(kmer)
    # M = []
    # with open('%s' %inName) as f:
    #     for line in f.readlines():
    #         if line.startswith('*'):
    #             break
    #         else:
    #             M.append(float(line.strip()))



    #add start and end node to graph
    Network.AddSTtoGraph(graph)

    predictedGenomes = []
    predictedCovs = []
    RemainedFlows = copy.copy(M)
    for flow in M:  
        RemainedFlows.remove(flow)
        #print "flow:", flow
        #calculate the nodes probability for current flow
        Network.CalculateNodeCost(graph, RemainedFlows, flow)
        path, cost = Network.FindLowCostPath(graph, flow)
        lowCostPathMeanCost = cost / len(path)
        #print "mean cost of low cost path found:", lowCostPathMeanCost
        #reduce the capacity by current flow
        Network.ReduceFlowFromPath(graph, path, flow)#, pNodeIndices)

        predictedGenomes.append(set(path))
        predictedCovs.append(flow)

        #weakly connected means the graph has been break.
        CCsCount = nx.number_weakly_connected_components(graph)
        if (CCsCount > 1):
            break

    #get the mismatch in segregating sites only
    predictedPolym = []
    for pgPath in predictedGenomes:
        polyms = []
        pNodeMismatches = []
        for nid in pgPath:
            if nid in pNodeIndices:
                pNodeMismatches.append(graph.node[nid]['mismatches'])
                pNodeMismatches.append(graph.node[nid]['mismatches'])
        mismatches = [item for item in pNodeMismatches if len(item) > 0]
        
        mismatchesStr = []
        for mlist in mismatches:            
            for p, m in mlist.iteritems():
                mismatchesStr.append(str.upper(m) + "," + str(p))
        predictedPolym.append(mismatchesStr)
        
        
    f = open(predictionsFile, "w")
    for i in range(0, len(predictedCovs)):
        pc = predictedCovs[i]
        f.write(str(pc) + ":")
        f.write("*".join([item for item in predictedPolym[i]]) + "\n")
        
    
main()
